@@ -8,6 +8,7 @@ import (
 	"runtime"
 	"strings"
 
+	"github.com/lmorg/murex/app"
 	"github.com/lmorg/murex/builtins/pipes/term"
 	"github.com/lmorg/murex/lang"
 	"github.com/lmorg/murex/lang/ref"
@@ -17,6 +18,7 @@ import (
 	"github.com/lmorg/murex/utils/ansi"
 	"github.com/lmorg/murex/utils/consts"
 	"github.com/lmorg/murex/utils/posix"
+	"github.com/lmorg/murex/utils/semver"
 )
 
 // Module is the structure for each module within a module's directory.
@@ -41,16 +43,18 @@ type Module struct {
 // Dependencies is a list of executables required by the module plus a list of
 // OSs the module is expected to work against
 type Dependencies struct {
-	Optional []string
-	Required []string
-	Platform []string
+	Optional     []string
+	Required     []string
+	Platform     []string
+	MurexVersion string
 }
 
 // Package is some basic details about the package itself as seen in the
 // package.json file located at the rood directory inside the package itself
 type Package struct {
-	Name    string
-	Version string
+	Name         string
+	Version      string
+	Dependencies Dependencies
 }
 
 var (
@@ -166,6 +170,17 @@ func (m *Module) checkDependencies() error {
 checkDeps:
 	var message string
 
+	// check supported version
+	if m.Dependencies.MurexVersion != "" {
+		ok, err := semver.Compare(app.Version(), m.Dependencies.MurexVersion)
+		if err != nil {
+			message += "  * Error checking supported Murex version: " + err.Error()
+		} else if !ok {
+			message += "  * This module is not supported for this version of Murex"
+		}
+	}
+
+	// check dependencies
 	for _, cmd := range m.Dependencies.Required {
 		if !(*autocomplete.GlobalExes.Get())[cmd] && lang.GoFunctions[cmd] == nil && !lang.MxFunctions.Exists(cmd) {
 			message += "  * Missing required executable, builtin or murex function: `" + cmd + "`" + utils.NewLineString
